% Quellcode der M-File Level-2 S-Funktion pt1sfun_m2.m 

function pt1sfun_m2(block)

% PT1SFUN_M2 modelliert das Verhalten eines PT1-Gliedes. Die Zeit-
% konstante T wird als Parameter bergeben
                          
setup(block);
  
%endfunction

function setup(block) 

  %%%% Beginn Block-Info %%%%

  % Definition des zugehrigen Zustandssystems:
  % (block.DialogPrm(1).Data entspricht dem Parameter T)
  A = [-1/block.DialogPrm(1).Data];  % Zustandsmatrix (hier skalar)
  B = [ 1/block.DialogPrm(1).Data];  % Eingangsmatrix (hier skalar)
  C = [ 1  ];                        % Ausgangsmatrix (hier skalar)
  
  % Anzahl der Ein- und Ausgnge des Blocks "M-File (level-2) S-Function"
  block.NumInputPorts  = 1;
  block.NumOutputPorts = 1;

  % Eigenschaften (Breite,Datentyp, Komplexitt etc.) des Ein- und Ausgangs 
  % sollen dynamisch bezogen werden
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;
 
  % Explizites Setzen bestimmter Eigenschaften
  block.InputPort(1).Dimensions        = size(B,1);
  block.InputPort(1).DirectFeedthrough = false; % kein direct feedthrough,
                                                % da D = 0 (siehe unten)
  
  block.OutputPort(1).Dimensions       = size(C,1);

  % Anzahl der bergebenen Parameter
  block.NumDialogPrms     = 1;              % Nur T wird bergeben
  block.DialogPrmsTunable = {'Nontunable'}; % T soll whrend der Simulation
                                            % nicht verstellbar sein

  % Abtastzeit und Offset werden zu Null gesetzt, da System
  % zeitkontinuierlich gerechnet wird
  block.SampleTimes = [0 0];
  
  % Anzahl der kontinuierlichen Zustandsgren
  block.NumContStates = size(A,1);
  
  %%%% Ende Block-Info %%%%

  % Bentigte Routinen in dieser S-Funktion
  block.RegBlockMethod('CheckParameters', @CheckPrms);
  block.RegBlockMethod('InitializeConditions', @InitializeConditions);
  block.RegBlockMethod('Outputs', @Outputs);
  block.RegBlockMethod('Derivatives', @Derivatives);
  
%endfunction

% berprfe bergebenen Parameter 
function CheckPrms(block)

  if block.DialogPrm(1).Data <= 0 
    error('Die Zeitkonstante T muss grer Null gewhlt werden');
  end
  
%endfunction

% Setze Anfangswert der Zustandsgre
function InitializeConditions(block)

  A = [-1/block.DialogPrm(1).Data];  % Zustandsmatrix (hier skalar)
  
  block.ContStates.Data(1) = zeros(size(A,1),1);
  
%endfunction

% Berechne Systemausgang zu y = C*x+D*u
function Outputs(block)

  x = block.ContStates.Data(1);
  u = block.InputPort(1).Data;
  C = [ 1  ];                        % Ausgangsmatrix (hier skalar)
  D = [ 0  ];                        % Durchschaltmatrix (hier skalar und
                                     % gleich Null, da sich beim PT1
                                     % der Eingang nicht direkt auf
                                     % den Ausgang auswirkt. 
  block.OutputPort(1).Data = C*x + D*u;
  
%endfunction


% Berechne Zustandsableitung zu x' = A*x+B*u
function Derivatives(block)

   x = block.ContStates.Data(1);
   u = block.InputPort(1).Data;
   A = [-1/block.DialogPrm(1).Data]; % Zustandsmatrix (hier skalar)  
   B = [ 1/block.DialogPrm(1).Data]; % Eingangsmatrix (hier skalar)

block.Derivatives.Data(1) = A*x + B*u;

%endfunction
 
